/**
 * Angular App Config Service.
 *
 * @file Main Angular's application service provider.
 */
import { Injectable, NgZone } from '@angular/core';
import { HttpClient }         from '@angular/common/http';
import 'rxjs/add/operator/take';
import { ProductModel }       from './data.model';
import { CouponModel }        from './data.model';
import { HttpHeaders }        from '@angular/common/http';
import { BehaviorSubject }    from 'rxjs/BehaviorSubject';

//Services
import { BoxtopService }      from './boxtop.service';

// Window global declaration
function _window (): any {
    return window;
}

// Jquery declaration
declare let $: any;

@Injectable ()
export class AppConfigService {

    private rewardGridPath: string = '/content/www/{banner}/en/justforyou/rewards/rewards-details';
    private objCoupons= {};
    private _arrPromoRule = [];
    public shopCouponRefreshSource = new BehaviorSubject<boolean> ( false );
    public shopCouponRefresh = this.shopCouponRefreshSource.asObservable ();

    public rewardsRefreshSource = new BehaviorSubject<boolean> ( false );
    public rewardsRefresh = this.rewardsRefreshSource.asObservable ();

    public postUserInfoSource = new BehaviorSubject<String> ( "" );
    public userInfoRefresh = this.postUserInfoSource.asObservable ();

    constructor ( private http: HttpClient, zone: NgZone, public objBoxTop: BoxtopService ) {
        this._arrPromoRule = _window ().SWY.CONFIGSERVICE.getConfigforPromoDesc ();
        window[ 'appConfigServiceRef' ] = {
            zone: zone,
            service: this,
        };

        this.checkBoxTopRegistered();
    }

    /**
     * Gets all hidden inputs to bring data into Angular application.
     *
     * @see app.module.ts
     */
    public getHiddenInputs (): void {
        const result: any = '';
        const wrapper: any = $ ( '.js-gridDataParse' );

        $ ( 'input[type="hidden"]', wrapper ).each ( function ( i, el ) {
            const $el = $ ( el );
            const val = $el.val ();
            const elName = $el.attr ( 'name' );

            if ( !val )
                return;

            switch ( elName ) {
                case 'gridDataSource':
                    window[ 'AB' ][ elName ] = JSON.parse ( val );
                    break;
                case 'availableSorts':
                    window[ 'AB' ][ elName ] = [];
                    val.split ( ',' ).map ( function ( elVal: any ) {
                        window[ 'AB' ][ elName ].push ( elVal.trim () );
                    } );
                    break;
                default:
                    window[ 'AB' ][ elName ] = val.toString ();
                    break;
            }
        } );
    }

    public getPromorulesArray () {
        return this._arrPromoRule;
    }

    public getBogoLabel (): String {
        return _window ().SWY.CONFIGSERVICE.getBannerConventions ().buy_one_get_one_free;
    }

    public getPromoDescription (): String {
        return _window ().SWY.CONFIGSERVICE.getBannerConventions ().promoDescription;
    }

    public getPromoDescriptionAPI ( product ) {
        switch ( product.promoType ) {
            case 'P':
                product[ 'promoTitle' ] = this.getPromoDescription ();
                product[ 'offerText' ] = '';
                break;
            case 'B':
            case 'M':
                product[ 'promoTitle' ] = this.getBogoLabel ();
                product[ 'offerText' ] = product[ 'promoText' ];
                break;
        }
    }

    public getAppContext () {
        return _window ().AB;
    }

    public getProductDetailPath (): string {
        return _window ().AB.htmlProductDetailPath;
    }

    public getCouponDetailPathShop (): string {
        return _window ().AB.htmlOfferDetailPathShop;
    }

    public getCouponDetailPathWww (): string {
        return _window ().AB.htmlOfferDetailPathWww;
    }

    public getRewardGridPathWww (): string {
        return _window ().AB.htmlRewardDetailPathWww;
    }

    public promoDescription ( product ) {

        const field = 'promoDescription';
        const newField = 'promoDescriptionVal';
        const priceColor = 'priceColor';

        const strPromoDesc = product.promoDescription;
        let found: Boolean = false;

        if ( !strPromoDesc ) {
            return;
        }

        console.log ( 'PRODUCT DESCRIPTION - DATA: ' + strPromoDesc );

        for ( let i = 0; i < this._arrPromoRule.length && found === false; i++ ) {
            const objRule = this._arrPromoRule[ i ];
            if ( strPromoDesc.match ( objRule[ 'rule' ] ) ) {
                product[ newField ] = objRule[ 'value' ];
                product[ priceColor ] = objRule[ 'priceColor' ];
                product[ 'offerText' ] = objRule[ 'offerText' ];
                found = true;
                objRule[ 'showStrikeThrough' ] == true && this.checkStrikeThrough ( product );
                console.log ( 'PRODUCT DESCRIPTION - MODIFIED: ' + product[ newField ] );
            }
        }

        if ( found === false ) {
            product[ newField ] = _window ().SWY.CONFIGSERVICE.getBannerConventions ().promoDescription;
            console.log ( 'PRODUCT DESCRIPTION - DEFAULT: ' + product[ newField ] );
            product[ priceColor ] = true;
            this.checkStrikeThrough ( product );
            product[ 'offerText' ] = '';
        }
    }

    public checkStrikeThrough ( product ) {
        if ( product[ 'promoType' ] == 'P' && product[ 'basePrice' ] != null && product[ 'price' ] > 0.00 && product[ 'basePrice' ] > 0.00 && product[ 'price' ] < product[ 'basePrice' ] ) {
            product[ 'showStrikeThrough' ] = true;
        }
        else {
            product[ 'showStrikeThrough' ] = false;
        }
    }

    public getJ4UCoupons () {
        return JSON.parse ( localStorage.getItem ( 'abCoupons' ) );
    }

    public getCoupons ( product ) {

        let offers = 0, offersClipped = 0;
        let offersNotAvailable = false;
        let offerName = '';
        let offersArray = [], newOffersArray = [];

        if ( product[ 'upc' ] != undefined ) {
            while ( product[ 'upc' ].split ( '' ).shift () === '0' ) {
                product[ 'upc' ] = product[ 'upc' ].substring ( 1, product[ 'upc' ].length );
            }
            // console.warn('WINDOW', _window().AB.COUPON.getCouponByUpc);
            // let arrCoupon: any = _window().AB.COUPON.getCouponByUpc(product['upc']);  -- this was the good one with
            // coupon js
            let arrCoupon: any = this.getCouponByUpc ( product[ 'upc' ] );
            // console.log(arrCoupon);

            if ( arrCoupon.length > 0 ) {

                for ( let i = 0; i < arrCoupon.length; i++ ) {
                    offers++;
                    offerName = arrCoupon[ i ];
                    offersArray.push ( arrCoupon[ i ] );
                }
                product[ 'offerCounter' ] = offers;

                for ( let offer in offersArray ) {
                    if ( offersArray[ 0 ].status == 'C' ) {
                        offersClipped++;
                        offersClipped < ( offersArray.length - 1 ) ? offersNotAvailable = true : offersNotAvailable = false;
                    }
                }
                product[ 'offersClipped' ] = offersClipped;
                if ( arrCoupon.length > 1 ) {
                    product[ 'moreCouponText' ] = "View All Coupons";
                    product[ 'clipStatus' ] = arrCoupon[ 0 ].status;
                    // product['couponsAvailable'] = "sd";
                }
                else {
                    let couponType = this.getCouponType ( arrCoupon[ 0 ].offerType );
                    console.log("couponType",couponType);
                    product[ 'couponText' ] = couponType + ": " + arrCoupon[ 0 ].offerPrice;
                    product[ 'couponCategory' ] = arrCoupon[ 0 ].category;
                    product[ 'showJ4U' ] = ( arrCoupon[ 0 ].offerType == 'PD' ) ? true : false;
                    product[ 'clipStatus' ] = arrCoupon[ 0 ].status;
                    product[ 'offerId' ] = arrCoupon[ 0 ].offerId;
                    console.log ( 'PRODUCT COUPON OFFER ID ' + arrCoupon[ 0 ].offerId );
                }
            }
        }
    }

    public clipCoupon ( couponId ) {
        console.log ( 'Clip the Coupon' );
        _window ().AB.COUPON.clipCoupon ( couponId );
    }

    public isLSCompatible () {
        let test = 'test';
        try {
            if (typeof window['AB'].isLSEnabled == 'undefined') {
                localStorage.setItem ( test, test );
                localStorage.removeItem ( test );
                console.log ( "==== LOCAL STORAGE IS ENABLED ====" );
                window['AB'].isLSEnabled = true;
            }
        }
        catch ( e ) {
            console.log ( "==== LOCAL STORAGE IS DISABLED ====" );
            window['AB'].isLSEnabled = false;
        }
        return window['AB'].isLSEnabled;
    }

    public _hasLSCoupons () {
        let objCoupons = this._get ();
        let response = ( objCoupons &&
                 objCoupons[ "offers" ] &&
                 objCoupons[ "offersByUpc" ] ) ? true : false;
        if(response) {
            if(objCoupons['updatedTime'] != undefined) {
                let updatedTime = Date.parse(JSON.parse(objCoupons['updatedTime']));
                let currentTime = Date.parse(JSON.parse(JSON.stringify(new Date())));
                let timeDiff = (currentTime - updatedTime) / 60000;
                console.log(updatedTime, response && timeDiff);
                response = (response && timeDiff < 120) ? true : false;
            }else {
                response = false;
            }
        }
        return response;
    }

    public _hasLSRewards () {
        let objRewards = this._getRewards ();
        return ( objRewards &&
                 objRewards[ "grOffers" ] ) ? true : false;
    }

    private _get ():any {
        let objCoupons = this.objCoupons;
        if(this.isLSCompatible () && Object.keys(objCoupons).length === 0) {
            let strCoupons = localStorage.getItem ( "abCoupons" );
            try {
                if(strCoupons) {
                    objCoupons = JSON.parse ( strCoupons );
                }
            }
            catch ( e ) {
                console.error ( "ERROR: COUPON - GETTING FROM LOCAL STORAGE" );
                console.error ( e );
            }
        }
       return objCoupons;
    }

    private _getRewards () {
        let strRewards = localStorage.getItem ( "abRewards" );
        let objRewards = {};
        try {
            objRewards = JSON.parse ( strRewards );
        }
        catch ( e ) {
            console.error ( "ERROR: REWARDS - GETTING FROM LOCAL STORAGE" );
            console.error ( e );
        }

        return objRewards;
    }

    public _hasLSDeals () {
        let objDeals = this._getDeals ();
        return objDeals ? true : false;
    }

    private _getDeals () {
        let strDeals = localStorage.getItem ( "abDeals" );
        let objDeals = {};
        try {
            objDeals = JSON.parse ( strDeals );
        }
        catch ( e ) {
            console.error ( "ERROR: DEAL - GETTING FROM LOCAL STORAGE" );
            console.error ( e );
        }

        return objDeals;
    }

    public loadRemoteCoupons () {
        let couponPath = '';

        if ( _window ().AB.COMMON.authorize () ) {
            couponPath = _window ().AB.offerRegisterPath;
        }
        else {
            couponPath = _window ().AB.offerGuestPath;
        }

        couponPath += '?offerPgm=PD-CC';
        couponPath += '&storeId=' + _window ().AB.userInfo.branchId;
        couponPath += '&transformOfferbyUpc=y';

        let httpOptions = { headers: new HttpHeaders ( _window ().AB.j4uHttpOptions ) };

        this.http.get ( couponPath, httpOptions )
            .take ( 1 )
            .subscribe ( ( objResponse ) => {
                console.log ( 'REMOTE COUPONS - ', objResponse );
                let objCoupons = objResponse;
                objCoupons[ 'branchId' ] = _window ().AB.userInfo.branchId;
                this._set( objCoupons );
                console.log ( "COUPON - FETCHED REMOTE STORAGE AND ADDED LOCAL STORAGE" );
                this.initCouponRelatedFunc ();
            } );
    }

    public isStoreChange (): any {
        let objCoupon = this._get ();
        return ( objCoupon[ 'branchId' ] == undefined || objCoupon[ 'branchId' ] != _window ().AB.userInfo.branchId );
    }

    private _set ( coupons ) {
        coupons['updatedTime'] = JSON.stringify(new Date());
        if(this.isLSCompatible()){
            localStorage.setItem ( "abCoupons", JSON.stringify ( coupons ) );
        } else {
            this.objCoupons = coupons;
        }
    }

    public initCouponRelatedFunc () {
        this.displayCouponInFullCart ();
        this.shopCouponRefreshSource.next ( true );
    }

    public isEmpty ( obj ) {
        // null and undefined are "empty"
        if ( obj == null ) return true;

        // Assume if it has a length property with a non-zero value
        // that that property is correct.
        if ( obj.length > 0 ) return false;
        if ( obj.length === 0 ) return true;

        // If it isn't an object at this point
        // it is empty, but it can't be anything *but* empty
        // Is it empty?  Depends on your application.
        if ( typeof obj !== "object" ) return true;

        return true;
    }

    ///////////Get COUPON by UPC
    public getCouponByUpc(upc) {
        var arrCoupons = [];


        if (this._hasLSCoupons()) {
            let objCoupons = this._get();
            var arrCouponId = objCoupons.offersByUpc[upc];
            if (arrCouponId != undefined) {
                for (var index in arrCouponId) {
                    var strCouponId = arrCouponId[index];
                    var objCoupon = objCoupons.offers[strCouponId];

                    if (objCoupon != undefined) {
                        objCoupon['offerPgmLabel'] = this.getCouponType(objCoupon.offerPgm);
                        objCoupon['offerId'] = objCoupon.offerId;
                        objCoupon['status'] = (objCoupon['deleted']) ? 'U' : objCoupon['status'];
                        arrCoupons.push(objCoupon);
                    }
                }
            }
        }
        return arrCoupons;
    }

    /**
     * Check Unattended Delivery and restricted items.
     *
     * @returns {boolean} It's true if item is restricted and false if it's not restricted.
     */
    public checkUnAttendedDelivery ( product?: ProductModel ): boolean {
        if ( window[ 'AB' ].COMMON.checkPendingOrder () && window[ 'AB' ].COMMON.checkUnAttendedDelivery () && product[ 'restrictedValue' ] && product[ 'restrictedValue' ] > 0 )
            return true;
        else
            return false;
    }

    /**
     * Check if provided UPC has coupons in LocalStorage and returns true or folse
     *
     * @param {string} upc Product unique upc code.
     */
    public checkHasCouponsByUPC ( upc ): boolean {
        if ( !upc )
            return false;

        let couponsInLStorage: any = '';
        let couponsThisUpc: any = '';

        couponsInLStorage = this._get();
        if ( !couponsInLStorage || !couponsInLStorage[ 'offersByUpc' ] )
            return false;

        couponsThisUpc = couponsInLStorage[ 'offersByUpc' ][ upc ];
        if ( !couponsThisUpc )
            return false;

        return ( couponsThisUpc.length > 0 );
    }

    /**
     *
     *
     *
     */
    public getCouponByAisleName ( aisleName: String ): any {
        let arrCoupons = [];
        let objCoupons = this._get();
        if ( !this.isEmpty ( aisleName ) && objCoupons.offers ) {
            aisleName = aisleName.toLowerCase ();
            for ( let index in objCoupons.offers ) {
                let objOffer = objCoupons.offers[ index ];
                if ( objOffer.category.toLowerCase () === aisleName ) {
                    objOffer[ 'offerPgmLabel' ] = this.getCouponType ( objOffer.offerPgm );
                    arrCoupons.push ( objOffer );
                }
            }
        }
        return arrCoupons;
    }


    public getClippedCouponByUpc ( upc: string ) {
        let allCoupons = [];
        let clippedCoupons = [];
        console.log ( 'UPC: ' + upc );
        let objCoupons = this._get();
        if (objCoupons.offersByUpc && objCoupons.offers) {

            let couponIds = objCoupons.offersByUpc[ upc ];
            if ( couponIds != undefined ) {
                allCoupons = couponIds.map ( x => {
                    let objCoupon = objCoupons.offers[ x ];
                    objCoupon[ 'offerPgmLabel' ] = this.getCouponType ( objCoupon[ 'offerPgm' ] );
                    return objCoupon;
                } ).filter ( x => x != undefined );

                clippedCoupons = allCoupons.filter ( x => x.status == 'C' );
            }
        }


        return {
            totalCoupons: allCoupons.length,
            totalClippedCoupons: clippedCoupons.length,
            clippedCoupons: clippedCoupons,
        };
    }

    public isClipped ( offerId ) {
        let response = false;
        console.log ( 'Offer: ' + offerId );
        if ( this._hasLSCoupons () ) {
            let objCoupons = this._get();
            if(objCoupons.offers[ offerId ]) {
                let objCoupon = objCoupons.offers[ offerId ];
                objCoupon.status = (objCoupon.deleted) ? 'U' : objCoupon.status; //get status using the deleted flag
                response = ( objCoupon != undefined && objCoupon.status == 'C' );
            }
        }
        return response;
    }

    public displayCouponInFullCart () {
        let fullCartTotalCoupons = 0;
        let fullCartClippedCoupons = 0;
        let that = this;
        $ ( '.j4u-clippedcoupons' ).each ( function ( index ) {
            let strUpc = $ ( this ).attr ( 'data-upc' );

            let objCoupon = that.getClippedCouponByUpc ( strUpc );
            let arrCoupon = objCoupon.clippedCoupons;
            fullCartTotalCoupons += objCoupon.totalCoupons;
            fullCartClippedCoupons += objCoupon.totalClippedCoupons;
            if ( arrCoupon.length > 0 ) {
                $ ( '.id-cartimage' ).eq ( index ).prepend ( '<span class="svg-icon just-for-you-16px icon-u-absolute-full-cart"></span>' );
            }

            for ( let cIndex in arrCoupon ) {
                let hCoupon = $ ( '<div></div>' );
                hCoupon.addClass ( 'clipped-coupon-wrapper' );
                hCoupon.append ( '<span class="svg-icon coupon-tick-mark"></span>' );
                hCoupon.append ( '<span class="clipped-coupon-text">' + arrCoupon[ cIndex ].offerPgmLabel + ": " + arrCoupon[ cIndex ].offerPrice + "</span>" );
                $ ( this ).append ( hCoupon );
            }
        } );
        window[ 'AB' ].DATALAYER.updateCartSavings ( fullCartTotalCoupons, fullCartClippedCoupons );
    }

    public getUpcByOffers ( offerId ) {
        let arrUpc = [];
        let objCoupons = this._get();
        if (objCoupons['offers'] && objCoupons['offers'] != null && objCoupons['offers'][ offerId ] ) {
            for ( let index in objCoupons['offersByUpc'] ) {
                let arrOfferId = objCoupons['offersByUpc'][ index ];
                if ( arrOfferId.indexOf ( offerId ) != -1 ) {
                    arrUpc.push ( index );
                }
            }
        }
        console.log ( 'getUpcByOffers: Returning array of UPC ', arrUpc );

        return arrUpc;
    }

    public getOffer(offerId) {
        let objOffers = '';
        let objCoupons = this._get();
        if (objCoupons.offers && objCoupons.offers[ offerId ] ) {
            objOffers = objCoupons.offers[ offerId];
        }

        return objOffers;
    }

    /**
     * Get coupon type mapping.
     *
     * @param {string} type Coupon type: MF, PD, CC, SC.
     */
    public getCouponType ( type: string ): string {
        let couponTypes = window[ 'SWY' ].CONFIGSERVICE.nimbusConfig.couponTypes;
        return ( couponTypes[ type ] == undefined ) ? '' : couponTypes[ type ];
    }

    public prop65WarningMessage ( product ) {

        let prop65Details = ( window[ 'SWY' ][ 'CONFIGSERVICE' ].getProp65Details () );
        product[ 'prop65WarningTextDetails' ] = "";

        if ( prop65Details.prop65Flag ) {

            product[ 'prop65Title' ] = "";
            product[ 'prop65PdpTitle' ] = "";

            if ( product[ 'prop65WarningText' ] != null && product[ 'prop65WarningText' ] != "" ) {
                product[ 'prop65Title' ] = prop65Details.prop65Title;
                product[ 'prop65PdpTitle' ] = prop65Details.prop65PDPTitle;
                product[ 'prop65WarningTextDetails' ] = product[ 'prop65WarningText' ];

            }

        }
    }

    public initServiceCall() {
        console.log("ANGULAR - REQUESTED SERVICE CALL");
        this.postUserInfoSource.next("initAngular");
    }

    public updateEcomGalleryCoupon(couponId) {
        let objOffer = this.getOffer(couponId);

        if(objOffer != "") {
            let allCoupons = JSON.parse(localStorage.getItem('abCoupons'));
            allCoupons.offers[couponId].status = 'C';
            allCoupons.offers[couponId].deleted = false;  //set deleted flag in localStorage
            localStorage.setItem('abCoupons', JSON.stringify(allCoupons));
        }
    }

    public checkBoxTopRegistered = function () {
        if (window['AB'].isWWW && window["AB"].COMMON.authorizeJ4U()) {
            this.userInfoRefresh.subscribe((data) => {
                if (data == 'initFunc') {
                    if (window['AB'].isWWW && window["AB"].COMMON.authorizeJ4U() && (!window["AB"].userInfo.boxTopCalled)) {
                        window["AB"].userInfo.boxTopCalled = true;
                        if (localStorage.getItem('loyaltyPartners')) {
                            const boxtopData = JSON.parse(localStorage.getItem('loyaltyPartners'));
                            if (boxtopData['loyaltyPartners']) {
                                boxtopData['loyaltyPartners'].forEach(function (loyaltyPartner) {
                                    if ((loyaltyPartner) && (loyaltyPartner.name == 'GeneralMills') && (loyaltyPartner.planNumber) && (loyaltyPartner.planNumber.length > 1)) {
                                        window['AB'].userInfo.boxTopAuth = true;
                                    }
                                });
                            }
                        } else {
                            this.objBoxTop.getGeneralMillsId().subscribe(boxtopData => {
                                localStorage.setItem('loyaltyPartners', JSON.stringify(boxtopData));
                                if (boxtopData['loyaltyPartners']) {
                                    boxtopData['loyaltyPartners'].forEach(function (loyaltyPartner) {
                                        if ((loyaltyPartner) && (loyaltyPartner.name == "GeneralMills") && (loyaltyPartner.planNumber) && (loyaltyPartner.planNumber.length > 1)) {
                                            window["AB"].userInfo.boxTopAuth = true;
                                        }
                                    });
                                }
                            });
                        }
                    }
                }
            });
        }
    }
}

