import {Injectable} from '@angular/core';
import {BehaviorSubject} from 'rxjs/BehaviorSubject';
import {isNumber} from "util";

declare var $: any;

@Injectable()
export class CouponFilterService {

    /* Source Params */

    private couponCategorySource = new BehaviorSubject<any[]>([]);
    currentCouponCategory = this.couponCategorySource.asObservable();

    private couponEventSource = new BehaviorSubject<any[]>([]);
    currentCouponEvent = this.couponEventSource.asObservable();

    private couponOfferTypeSource = new BehaviorSubject<any[]>([]);
    currentCouponOfferType = this.couponOfferTypeSource.asObservable();

    private couponAisleTypeSource = new BehaviorSubject<any[]>([]);
    currentCouponAisleType = this.couponAisleTypeSource.asObservable();

    /* Filter Source Params */

    private filterCategoryParamSource = new BehaviorSubject<String>('');
    currentFilterCategory = this.filterCategoryParamSource.asObservable();

    private filterEventParamSource = new BehaviorSubject<String>('');
    currentFilterEvent = this.filterEventParamSource.asObservable();

    private filterOfferTypeParamSource = new BehaviorSubject<String>('');
    currentFilterOfferType = this.filterOfferTypeParamSource.asObservable();

    private filterAisleTypeParamSource = new BehaviorSubject<String>('');
    currentFilterAisleType = this.filterAisleTypeParamSource.asObservable();

    private addFilterSource = new BehaviorSubject('');

    currentFilterAdded = this.addFilterSource.asObservable();

    public addAllFilters = new BehaviorSubject(false);

    public offerTypeFilterCounter = new BehaviorSubject<number>(0);

    currentOfferTypeFilterCounter = this.offerTypeFilterCounter.asObservable();

    public categoryFilterCounter = new BehaviorSubject<number>(0);

    currentCategoryFilterCounter = this.categoryFilterCounter.asObservable();

    public eventsFilterCounter = new BehaviorSubject<number>(0);

    currentEventsFilterCounter = this.eventsFilterCounter.asObservable();

    public _couponCategories: any = [];
    public _couponEvents: any = [];
    public _couponOfferTypes: any = [];
    public _couponAisleTypes: any = [];

    public constructor() {
    }

    /* Add Category Filter */
    public addCouponCategory(value) {
        let queryString = encodeURIComponent(value);
        let isValid = this.isExist(this._couponCategories, queryString);
        if (isValid.length == 0) {
            let objValue = {'query': queryString, 'text': value};
            this._couponCategories.push(objValue);
            this.couponCategorySource.next(this._couponCategories);
        }
    }

    /* Add Event Filter */
    public addCouponEvent(value) {
        let queryString = "";
        queryString = encodeURIComponent(value);
        let isValid = this.isExist(this._couponEvents, queryString);
        if (isValid.length == 0) {
            let objValue = {'query': queryString, 'text': value};
            this._couponEvents.push(objValue);
            this.couponEventSource.next(this._couponEvents);
        }
    }

    /* Add Offer Type Filter */
    public addCouponOfferType(value) {
        let queryString = encodeURIComponent(value);
        let isValid = this.isExist(this._couponOfferTypes, queryString);
        if (isValid.length == 0) {
            let objValue = {'query': queryString, 'text': value};
            this._couponOfferTypes.push(objValue);
            this.couponOfferTypeSource.next(this._couponOfferTypes);
        }
    }

    /* Add Aisle Type Filter */
    public addCouponAisleType(value) {
        let queryString = encodeURIComponent(value);
        let isValid = this.isExist(this._couponAisleTypes, queryString);
        if (isValid.length == 0) {
            let objValue = {'query': queryString, 'text': value};
            this._couponAisleTypes.push(objValue);
            this.couponAisleTypeSource.next(this._couponAisleTypes);
        }
    }

    private isExist(arr: any, value: String) {
        return arr.filter(function (objValue) {
            return objValue.query.toLowerCase().indexOf(value.toLowerCase()) > -1;        
        });
    }

    /* Set Params for Filters */

    public addFilterCategoryParam(param) {
        this.filterCategoryParamSource.next(param);
    }

    public addFilterEventParam(param) {
        this.filterEventParamSource.next(param);
    }

    public addFilterOfferTypeParam(param) {
        this.filterOfferTypeParamSource.next(param);
    }

    public addFilterAisleTypeParam(param) {
        this.filterAisleTypeParamSource.next(param);
    }

    /* Count Category Filter Choices */

    public countCategoryFilters(counter : number) {
        this.categoryFilterCounter.next(counter);
    }

    /* Count Events Filter Choices */

    public countEventsFilters(counter : number) {
        this.eventsFilterCounter.next(counter);
    }

    /* Count Offer Type Filter Choices */

    public countOfferTypeFilters(counter : number) {
        this.offerTypeFilterCounter.next(counter);
    }

    /* Filter data pushState() */

    public applyFilter(message: string) {
        this.addFilterSource.next(message);
    }

    public applyFilterAll() {
        this.addAllFilters.next(true);
    }

    public clearAllFilters() {
        event.stopPropagation();
        window.history.replaceState({}, null, window.location.pathname);
        window.location.reload();
    }
}
